@@ -1,10 +1,12 @@
-__version__ = '1.0.1'
+__version__ = '1.0.4'
 
 from contextlib import contextmanager
+from zlib import crc32
 
 @contextmanager
 def advisory_lock(lock_id, shared=False, wait=True, using=None):
 
+    import six
     from django.db import DEFAULT_DB_ALIAS, connections, transaction
 
     if using is None:
@@ -34,12 +36,20 @@ def advisory_lock(lock_id, shared=False, wait=True, using=None):
         if len(lock_id) != 2:
             raise ValueError("Tuples and lists as lock IDs must have exactly two entries.")
 
-        if not isinstance(lock_id[0], (int, long,)) or not isinstance(lock_id[1], (int, long,)):
-            raise ValueError("Both members of a tuple/list lock ID must be ints or longs")
+        if not isinstance(lock_id[0], six.integer_types) or not isinstance(lock_id[1], six.integer_types):
+            raise ValueError("Both members of a tuple/list lock ID must be integers")
 
         tuple_format = True
-    elif not isinstance(lock_id, (int, long,)):
-        lock_id = long(lock_id.__hash__())
+    elif isinstance(lock_id, six.string_types):
+        # Generates an id within postgres integer range (-2^31 to 2^31 - 1).
+        # crc32 generates an unsigned integer in Py3, we convert it into
+        # a signed integer using 2's complement (this is a noop in Py2)
+        pos = crc32(lock_id.encode("utf-8"))
+        lock_id = (2**31 - 1) & pos
+        if pos & 2**31:
+            lock_id -= 2**31
+    elif not isinstance(lock_id, six.integer_types):
+        raise ValueError("Cannot use %s as a lock id" % lock_id)
 
     if tuple_format:
         base = "SELECT %s(%d, %d)"
@@ -55,7 +65,10 @@ def advisory_lock(lock_id, shared=False, wait=True, using=None):
 
     cursor.execute(command)
 
-    acquired = cursor.fetchone()[0]
+    if not wait:
+        acquired = cursor.fetchone()[0]
+    else:
+        acquired = True
 
     try:
         yield acquired
@@ -64,4 +77,6 @@ def advisory_lock(lock_id, shared=False, wait=True, using=None):
             release_params = ( release_function_name, ) + params
 
             command = base % release_params
-            cursor.execute(command)
\ No newline at end of file
+            cursor.execute(command)
+
+        cursor.close()
